@@ -921,6 +921,10 @@ bool IRBuilder::visit(QQmlJS::AST::UiPragma *node)
         } else if (node->name == "ValueTypeBehavior"_L1) {
             if (!PragmaParser<Pragma::ValueTypeBehaviorValue>::run(this, node, pragma))
                 return false;
+        } else if (node->name == "Translator"_L1) {
+            pragma->type = Pragma::Translator;
+            pragma->translationContextIndex = registerString(node->values->value.toString());
+
         } else {
             recordError(node->pragmaToken, QCoreApplication::translate(
                             "QQmlParser", "Unknown pragma '%1'").arg(node->name));
@@ -1641,10 +1645,14 @@ void QmlUnitGenerator::generate(Document &output, const QV4::CompiledData::Depen
 
     Unit *jsUnit = nullptr;
 
+    if (!output.javaScriptCompilationUnit)
+        output.javaScriptCompilationUnit.adopt(new QV4::CompiledData::CompilationUnit);
+
     // We may already have unit data if we're loading an ahead-of-time generated cache file.
-    if (output.javaScriptCompilationUnit.data) {
-        jsUnit = const_cast<Unit *>(output.javaScriptCompilationUnit.data);
-        output.javaScriptCompilationUnit.dynamicStrings = output.jsGenerator.stringTable.allStrings();
+    if (output.javaScriptCompilationUnit->unitData()) {
+        jsUnit = const_cast<Unit *>(output.javaScriptCompilationUnit->unitData());
+        output.javaScriptCompilationUnit->dynamicStrings
+                = output.jsGenerator.stringTable.allStrings();
     } else {
         Unit *createdUnit;
         jsUnit = createdUnit = output.jsGenerator.generateUnit();
@@ -1711,6 +1719,11 @@ void QmlUnitGenerator::generate(Document &output, const QV4::CompiledData::Depen
                     createdUnit->flags |= Unit::ValueTypesAddressable;
                 }
                 break;
+            case Pragma::Translator:
+                if (createdUnit->translationTableSize)
+                    if (quint32_le *index = createdUnit->translationContextIndex())
+                        *index = p->translationContextIndex;
+                break;
             }
         }
 
@@ -1912,7 +1925,7 @@ void QmlUnitGenerator::generate(Document &output, const QV4::CompiledData::Depen
         }
     }
 
-    if (!output.javaScriptCompilationUnit.data) {
+    if (!output.javaScriptCompilationUnit->unitData()) {
         // Combine the qml data into the general unit data.
         jsUnit = static_cast<QV4::CompiledData::Unit *>(realloc(jsUnit, jsUnit->unitSize + totalSize));
         jsUnit->offsetToQmlUnit = jsUnit->unitSize;
@@ -1945,8 +1958,8 @@ void QmlUnitGenerator::generate(Document &output, const QV4::CompiledData::Depen
         qDebug() << "    " << totalStringSize << "bytes total strings";
     }
 
-    output.javaScriptCompilationUnit.setUnitData(jsUnit, qmlUnit, output.jsModule.fileName,
-                                                 output.jsModule.finalUrl);
+    output.javaScriptCompilationUnit->setUnitData(
+            jsUnit, qmlUnit, output.jsModule.fileName, output.jsModule.finalUrl);
 }
 
 char *QmlUnitGenerator::writeBindings(char *bindingPtr, const Object *o, BindingFilter filter) const
