@@ -45,7 +45,7 @@ struct ControlFlow;
 struct ControlFlowCatch;
 struct ControlFlowFinally;
 
-class Q_QML_COMPILER_PRIVATE_EXPORT CodegenWarningInterface
+class Q_QML_COMPILER_EXPORT CodegenWarningInterface
 {
 public:
     virtual void reportVarUsedBeforeDeclaration(const QString &name, const QString &fileName,
@@ -60,7 +60,7 @@ inline CodegenWarningInterface *defaultCodegenWarningInterface()
     return &iface;
 }
 
-class Q_QML_COMPILER_PRIVATE_EXPORT Codegen: protected QQmlJS::AST::Visitor
+class Q_QML_COMPILER_EXPORT Codegen: protected QQmlJS::AST::Visitor
 {
 protected:
     using BytecodeGenerator = QV4::Moth::BytecodeGenerator;
@@ -731,8 +731,9 @@ public:
             const QString &name, bool lhs,
             const QQmlJS::SourceLocation &accessLocation = QQmlJS::SourceLocation());
 
-    QV4::CompiledData::CompilationUnit generateCompilationUnit(bool generateUnitData = true);
-    static QV4::CompiledData::CompilationUnit compileModule(
+    QQmlRefPointer<QV4::CompiledData::CompilationUnit> generateCompilationUnit(
+            bool generateUnitData = true);
+    static QQmlRefPointer<QV4::CompiledData::CompilationUnit> compileModule(
             bool debugMode, const QString &url, const QString &sourceCode,
             const QDateTime &sourceTimeStamp, QList<QQmlJS::DiagnosticMessage> *diagnostics);
 
@@ -854,7 +855,7 @@ private:
     void throwError(ErrorType errorType, const QQmlJS::SourceLocation &loc,
                     const QString &detail);
     bool traverseOptionalChain(QQmlJS::AST::Node *node);
-    void optionalChainFinalizer(Reference expressionResult, bool tailOfChain,
+    void optionalChainFinalizer(const Reference &expressionResult, bool tailOfChain,
                                 bool isDeleteExpression = false);
     Reference loadSubscriptForCall(const Reference &base);
     void generateThrowException(const QString &type, const QString &text = QString());
