@@ -82,6 +82,10 @@ class QQmlEngine;
 class QQmlCustomParser;
 class QQmlTypeNotAvailable;
 
+class QQmlV4Function;
+using QQmlV4FunctionPtr = QQmlV4Function *;
+using QQmlV4ExecutionEnginePtr = QV4::ExecutionEngine *;
+
 template<class T>
 QQmlCustomParser *qmlCreateCustomParser()
 {
@@ -366,8 +370,8 @@ namespace QQmlPrivate
         struct Properties<Parent, void>
         {
             using Func = QQmlAttachedPropertiesFunc<QObject>;
-            static const QMetaObject *staticMetaObject() { return nullptr; };
-            static Func attachedPropertiesFunc() { return nullptr; };
+            static const QMetaObject *staticMetaObject() { return nullptr; }
+            static Func attachedPropertiesFunc() { return nullptr; }
         };
 
         using Type = typename std::conditional<
@@ -722,6 +726,9 @@ namespace QQmlPrivate
         void initGetValueLookup(uint index, const QMetaObject *metaObject, QMetaType type) const;
 
         bool getEnumLookup(uint index, void *target) const;
+#if QT_QML_REMOVED_SINCE(6, 6)
+        bool getEnumLookup(uint index, int *target) const;
+#endif
         void initGetEnumLookup(uint index, const QMetaObject *metaObject,
                                const char *enumerator, const char *enumValue) const;
 
@@ -835,29 +842,6 @@ namespace QQmlPrivate
         return qstrcmp(metaObject->classInfo(index).value(), "true") == 0;
     }
 
-    inline const char *classElementName(const QMetaObject *metaObject)
-    {
-        const char *elementName = classInfo(metaObject, "QML.Element");
-        if (qstrcmp(elementName, "auto") == 0) {
-            const char *strippedClassName = metaObject->className();
-            for (const char *c = strippedClassName; *c != '\0'; c++) {
-                if (*c == ':')
-                    strippedClassName = c + 1;
-            }
-
-            return strippedClassName;
-        }
-        if (qstrcmp(elementName, "anonymous") == 0)
-            return nullptr;
-
-        if (!elementName) {
-            qWarning().nospace() << "Missing QML.Element class info \"" << elementName << "\""
-                                 << " for " << metaObject->className();
-        }
-
-        return elementName;
-    }
-
     template<class T, class = std::void_t<>>
     struct QmlExtended
     {
@@ -968,7 +952,7 @@ namespace QQmlPrivate
     };
 
     template<class T>
-    struct QmlInterface<T, std::void_t<typename T::QmlIsInterface>>
+    struct QmlInterface<T, std::void_t<typename T::QmlIsInterface, decltype(qobject_interface_iid<T *>())>>
     {
         static constexpr bool Value = bool(T::QmlIsInterface::yes);
     };
@@ -998,7 +982,7 @@ namespace QQmlPrivate
                 return std::is_copy_constructible_v<T>;
         }
 
-        static QMetaType self()
+        static constexpr QMetaType self()
         {
             if constexpr (std::is_base_of_v<QObject, T>)
                 return QMetaType::fromType<T*>();
@@ -1006,7 +990,7 @@ namespace QQmlPrivate
                 return QMetaType::fromType<T>();
         }
 
-        static QMetaType list()
+        static constexpr QMetaType list()
         {
             if constexpr (std::is_base_of_v<QObject, T>)
                 return QMetaType::fromType<QQmlListProperty<T>>();
@@ -1014,13 +998,28 @@ namespace QQmlPrivate
                 return QMetaType::fromType<QList<T>>();
         }
 
-        static QMetaSequence sequence()
+        static constexpr QMetaSequence sequence()
         {
             if constexpr (std::is_base_of_v<QObject, T>)
                 return QMetaSequence();
             else
                 return QMetaSequence::fromContainer<QList<T>>();
         }
+
+        static constexpr int size()
+        {
+            return sizeof(T);
+        }
+    };
+
+    template<>
+    struct QmlMetaType<void>
+    {
+        static constexpr bool hasAcceptableCtors() { return true; }
+        static constexpr QMetaType self() { return QMetaType(); }
+        static constexpr QMetaType list() { return QMetaType(); }
+        static constexpr QMetaSequence sequence() { return QMetaSequence(); }
+        static constexpr int size() { return 0; }
     };
 
     template<typename T, typename E, typename WrapperT = T>
@@ -1061,7 +1060,7 @@ namespace QQmlPrivate
             3,
             QmlMetaType<T>::self(),
             QmlMetaType<T>::list(),
-            int(sizeof(T)),
+            QmlMetaType<T>::size(),
             Constructors<T>::createInto,
             nullptr,
             ValueType<T, E>::create,
@@ -1157,4 +1156,7 @@ namespace QQmlPrivate
 
 QT_END_NAMESPACE
 
+Q_DECLARE_OPAQUE_POINTER(QQmlV4FunctionPtr)
+Q_DECLARE_OPAQUE_POINTER(QQmlV4ExecutionEnginePtr)
+
 #endif // QQMLPRIVATE_H
