@@ -15,11 +15,19 @@
 // We mean it.
 //
 
+#include "qmetatypesjsonprocessor_p.h"
+
 #include <QtCore/qversionnumber.h>
 
 QT_BEGIN_NAMESPACE
 
 QTypeRevision handleInMinorVersion(QTypeRevision revision, int majorVersion);
+QAnyStringView interfaceName(const Interface &iface);
+
+QDebug warning(const MetaType &classDef);
+QDebug warning(QAnyStringView fileName, int lineNumber = 0);
+
+QDebug error(QAnyStringView fileName, int lineNumber = 0);
 
 QT_END_NAMESPACE
 
