@@ -9,7 +9,7 @@
 #include <private/qquickcontext2dtexture_p.h>
 #include <private/qsgadaptationlayer_p.h>
 #include <qsgtextureprovider.h>
-#include <QtQuick/private/qquickpixmapcache_p.h>
+#include <QtQuick/private/qquickpixmap_p.h>
 #include <QtGui/QGuiApplication>
 #include <qsgtextureprovider.h>
 
@@ -252,7 +252,7 @@ QQuickCanvasItemPrivate::~QQuickCanvasItemPrivate()
     QPainter instead of the more expensive and likely less performing
     JavaScript and Context2D approach.
 
-    \sa Context2D, QQuickPaintedItem
+    \sa Context2D, QQuickPaintedItem, {Qt Quick Examples - Pointer Handlers}
 */
 
 QQuickCanvasItem::QQuickCanvasItem(QQuickItem *parent)
@@ -444,9 +444,8 @@ void QQuickCanvasItem::setCanvasWindow(const QRectF& rect)
     \qmlproperty enumeration QtQuick::Canvas::renderTarget
     Holds the current canvas render target.
 
-    \list
-    \li Canvas.Image  - render to an in memory image buffer.
-    \endlist
+    \value Canvas.Image                 Render to an in-memory image buffer.
+    \value Canvas.FramebufferObject     As of Qt 6.0, this value is ignored.
 
     This hint is supplied along with renderStrategy to the graphics context to
     determine the method of rendering. A renderStrategy, renderTarget or a
@@ -480,11 +479,9 @@ void QQuickCanvasItem::setRenderTarget(QQuickCanvasItem::RenderTarget target)
     \qmlproperty enumeration QtQuick::Canvas::renderStrategy
     Holds the current canvas rendering strategy.
 
-    \list
-    \li Canvas.Immediate - context will perform graphics commands immediately in the main UI thread.
-    \li Canvas.Threaded - context will defer graphics commands to a private rendering thread.
-    \li Canvas.Cooperative - context will defer graphics commands to the applications global render thread.
-    \endlist
+    \value Canvas.Immediate     context will perform graphics commands immediately in the main UI thread.
+    \value Canvas.Threaded      context will defer graphics commands to a private rendering thread.
+    \value Canvas.Cooperative   context will defer graphics commands to the applications global render thread.
 
     This hint is supplied along with renderTarget to the graphics context to
     determine the method of rendering. A renderStrategy, renderTarget or a
@@ -813,7 +810,7 @@ QSGTextureProvider *QQuickCanvasItem::textureProvider() const
 
 */
 
-void QQuickCanvasItem::getContext(QQmlV4Function *args)
+void QQuickCanvasItem::getContext(QQmlV4FunctionPtr args)
 {
     Q_D(QQuickCanvasItem);
 
@@ -857,7 +854,7 @@ void QQuickCanvasItem::getContext(QQmlV4Function *args)
     scene.
 */
 
-void QQuickCanvasItem::requestAnimationFrame(QQmlV4Function *args)
+void QQuickCanvasItem::requestAnimationFrame(QQmlV4FunctionPtr args)
 {
     QV4::Scope scope(args->v4engine());
     QV4::ScopedFunctionObject f(scope, (*args)[0]);
@@ -886,7 +883,7 @@ void QQuickCanvasItem::requestAnimationFrame(QQmlV4Function *args)
     This function will cancel the animation callback referenced by \a handle.
 */
 
-void QQuickCanvasItem::cancelRequestAnimationFrame(QQmlV4Function *args)
+void QQuickCanvasItem::cancelRequestAnimationFrame(QQmlV4FunctionPtr args)
 {
     QV4::Scope scope(args->v4engine());
     QV4::ScopedValue v(scope, (*args)[0]);
@@ -961,12 +958,12 @@ bool QQuickCanvasItem::save(const QString &filename, const QSizeF &imageSize) co
     return toImage(QRectF(QPointF(0, 0), imageSize)).save(url.toLocalFile());
 }
 
-QQmlRefPointer<QQuickCanvasPixmap> QQuickCanvasItem::loadedPixmap(const QUrl& url)
+QQmlRefPointer<QQuickCanvasPixmap> QQuickCanvasItem::loadedPixmap(const QUrl& url, QSizeF sourceSize)
 {
     Q_D(QQuickCanvasItem);
     QUrl fullPathUrl = d->baseUrl.resolved(url);
     if (!d->pixmaps.contains(fullPathUrl)) {
-        loadImage(url);
+        loadImage(url, sourceSize);
     }
     return d->pixmaps.value(fullPathUrl);
 }
@@ -980,7 +977,7 @@ QQmlRefPointer<QQuickCanvasPixmap> QQuickCanvasItem::loadedPixmap(const QUrl& ur
 */
 
 /*!
-    \qmlmethod QtQuick::Canvas::loadImage(url image)
+    \qmlmethod QtQuick::Canvas::loadImage(url image, size sourceSize = undefined)
 
     Loads the given \a image asynchronously.
 
@@ -989,10 +986,14 @@ QQmlRefPointer<QQuickCanvasPixmap> QQuickCanvasItem::loadedPixmap(const QUrl& ur
 
     \note Only loaded images can be painted on the Canvas item.
 
+    If \a sourceSize is specified, the image will be scaled to that size during loading. This is
+    useful for loading scalable (vector) images (eg. SVGs) at their intended display size. This
+    parameter was introduced in Qt 6.7.
+
     \sa unloadImage(), imageLoaded(), isImageLoaded(),
         Context2D::createImageData(), Context2D::drawImage()
 */
-void QQuickCanvasItem::loadImage(const QUrl& url)
+void QQuickCanvasItem::loadImage(const QUrl& url, QSizeF sourceSize)
 {
     Q_D(QQuickCanvasItem);
     QUrl fullPathUrl = d->baseUrl.resolved(url);
@@ -1004,6 +1005,8 @@ void QQuickCanvasItem::loadImage(const QUrl& url)
 
         pix->load(qmlEngine(this)
                 , fullPathUrl
+                , QRect()
+                , sourceSize.toSize()
                 , QQuickPixmap::Cache | QQuickPixmap::Asynchronous);
         if (pix->isLoading())
             pix->connectFinished(this, SIGNAL(imageLoaded()));
