@@ -130,8 +130,7 @@ void QQuickAbstractAnimationPrivate::commence()
     QQmlProperties properties;
 
     auto *newInstance = q->transition(actions, properties, QQuickAbstractAnimation::Forward);
-    // transition can return a nullptr; that's the only allowed case were old and new have the same value
-    Q_ASSERT(newInstance != animationInstance || !newInstance);
+    Q_ASSERT(newInstance != animationInstance);
     delete animationInstance;
     animationInstance = newInstance;
 
@@ -291,11 +290,6 @@ void QQuickAbstractAnimation::setRunning(bool r)
         // Therefore, the state of d->running will in that case be different than r if we are back in
         // the root stack frame of the recursive calls to setRunning()
         emit runningChanged(d->running);
-    } else if (d->animationInstance) {
-        // If there was a recursive call, make sure the d->running is set correctly
-        d->running = d->animationInstance->isRunning();
-    } else {
-        d->running = r;
     }
 }
 
@@ -1194,7 +1188,7 @@ QVariant QQuickPropertyAction::value() const
 void QQuickPropertyAction::setValue(const QVariant &v)
 {
     Q_D(QQuickPropertyAction);
-    if (d->value.isNull || d->value != v) {
+    if (!d->value.isValid() || d->value != v) {
         d->value = v;
         emit valueChanged(v);
     }
