@@ -47,6 +47,10 @@
 #include <QtWidgets/qgraphicsview.h>
 #endif
 
+#if QT_CONFIG(vulkan)
+#include <QtGui/private/qvulkandefaultinstance_p.h>
+#endif
+
 QT_BEGIN_NAMESPACE
 
 QQuickWidgetOffscreenWindow::QQuickWidgetOffscreenWindow(QQuickWindowPrivate &dd, QQuickRenderControl *control)
@@ -1023,13 +1027,10 @@ void QQuickWidgetPrivate::initializeWithRhi()
 {
     Q_Q(QQuickWidget);
 
-    QWidgetPrivate *tlwd = QWidgetPrivate::get(q->window());
     // when reparenting, the rhi may suddenly be different
     if (rhi) {
-        QRhi *tlwRhi = nullptr;
-        if (QWidgetRepaintManager *repaintManager = tlwd->maybeRepaintManager())
-            tlwRhi = repaintManager->rhi();
-        if (tlwRhi && rhi != tlwRhi)
+        QRhi *backingStoreRhi = QWidgetPrivate::rhi();
+        if (backingStoreRhi && rhi != backingStoreRhi)
             rhi = nullptr;
     }
 
@@ -1041,18 +1042,16 @@ void QQuickWidgetPrivate::initializeWithRhi()
         if (rhi)
             return;
 
-        if (QWidgetRepaintManager *repaintManager = tlwd->maybeRepaintManager()) {
-            rhi = repaintManager->rhi();
-            if (rhi) {
-                // We don't own the RHI, so make sure we clean up if it goes away
-                rhi->addCleanupCallback(q, [this](QRhi *rhi) {
-                    if (this->rhi == rhi) {
-                        invalidateRenderControl();
-                        deviceLost = true;
-                        this->rhi = nullptr;
-                    }
-                });
-            }
+        if (QRhi *backingStoreRhi = QWidgetPrivate::rhi()) {
+            rhi = backingStoreRhi;
+            // We don't own the RHI, so make sure we clean up if it goes away
+            rhi->addCleanupCallback(q, [this](QRhi *rhi) {
+                if (this->rhi == rhi) {
+                    invalidateRenderControl();
+                    deviceLost = true;
+                    this->rhi = nullptr;
+                }
+            });
         }
 
         if (!rhi) {
@@ -1084,6 +1083,8 @@ void QQuickWidgetPrivate::initializeWithRhi()
 #if QT_CONFIG(vulkan)
             if (QWindow *w = q->window()->windowHandle())
                 offscreenWindow->setVulkanInstance(w->vulkanInstance());
+            else if (rhi == offscreenRenderer.rhi())
+                offscreenWindow->setVulkanInstance(QVulkanDefaultInstance::instance());
 #endif
             renderControl->initialize();
         }
@@ -1130,7 +1131,7 @@ void QQuickWidget::createFramebufferObject()
 
     // Could be a simple hide - show, in which case the previous texture is just fine.
     if (!d->outputTexture) {
-        d->outputTexture = d->rhi->newTexture(QRhiTexture::RGBA8, fboSize, 1, QRhiTexture::RenderTarget);
+        d->outputTexture = d->rhi->newTexture(QRhiTexture::RGBA8, fboSize, 1, QRhiTexture::RenderTarget | QRhiTexture::UsedAsTransferSource);
         if (!d->outputTexture->create()) {
             qWarning("QQuickWidget: failed to create output texture of size %dx%d",
                      fboSize.width(), fboSize.height());
