@@ -1,5 +1,5 @@
 // Copyright (C) 2019 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtTest/QtTest>
 #include <QDir>
@@ -16,6 +16,25 @@
 
 using namespace QQmlJS::Dom;
 
+// TODO refactor extension helpers
+const QString QML_EXT = ".qml";
+const QString JS_EXT = ".js";
+const QString MJS_EXT = ".mjs";
+
+static QStringView fileExt(QStringView filename)
+{
+    if (filename.endsWith(QML_EXT)) {
+        return QML_EXT;
+    }
+    if (filename.endsWith(JS_EXT)) {
+        return JS_EXT;
+    }
+    if (filename.endsWith(MJS_EXT)) {
+        return MJS_EXT;
+    }
+    Q_UNREACHABLE();
+};
+
 class TestQmlformat: public QQmlDataTest
 {
     Q_OBJECT
@@ -27,6 +46,7 @@ public:
 private Q_SLOTS:
     void initTestCase() override;
 
+    //actually testFormat tests CLI of qmlformat
     void testFormat();
     void testFormat_data();
 
@@ -40,10 +60,19 @@ private Q_SLOTS:
 
     void testBackupFileLimit();
 
+    void testFilesOption_data();
+    void testFilesOption();
+
+    void plainJS_data();
+    void plainJS();
+
+    void ecmascriptModule();
+
 private:
     QString readTestFile(const QString &path);
+    //TODO(QTBUG-117849) refactor this helper function
     QString runQmlformat(const QString &fileToFormat, QStringList args, bool shouldSucceed = true,
-                         RunOption rOption = RunOption::OnCopy);
+                         RunOption rOption = RunOption::OnCopy, QStringView ext = QML_EXT);
     QString formatInMemory(const QString &fileToFormat, bool *didSucceed = nullptr,
                            LineWriterOptions options = LineWriterOptions(),
                            WriteOutChecks extraChecks = WriteOutCheck::ReparseCompare,
@@ -130,6 +159,14 @@ void TestQmlformat::initTestCase()
     m_invalidFiles << "tests/auto/qml/qqmllanguage/data/typeAnnotations.2.qml";
     m_invalidFiles << "tests/auto/qml/qqmlparser/data/disallowedtypeannotations/qmlnestedfunction.qml";
     m_invalidFiles << "tests/auto/qmlls/utils/data/emptyFile.qml";
+    m_invalidFiles << "tests/auto/qmlls/utils/data/completions/missingRHS.qml";
+    m_invalidFiles << "tests/auto/qmlls/utils/data/completions/missingRHS.parserfail.qml";
+    m_invalidFiles << "tests/auto/qmlls/utils/data/completions/attachedPropertyMissingRHS.qml";
+    m_invalidFiles << "tests/auto/qmlls/utils/data/completions/groupedPropertyMissingRHS.qml";
+    m_invalidFiles << "tests/auto/qmlls/utils/data/completions/afterDots.qml";
+    m_invalidFiles << "tests/auto/qmlls/modules/data/completions/bindingAfterDot.qml";
+    m_invalidFiles << "tests/auto/qmlls/modules/data/completions/defaultBindingAfterDot.qml";
+    m_invalidFiles << "tests/auto/qmlls/utils/data/qualifiedModule.qml";
 
     // Files that get changed:
     // rewrite of import "bla/bla/.." to import "bla"
@@ -341,6 +378,40 @@ void TestQmlformat::testFormat_data()
     QTest::newRow("ellipsisFunctionArgument")
             << "ellipsisFunctionArgument.qml"
             << "ellipsisFunctionArgument.formatted.qml" << QStringList{} << RunOption::OnCopy;
+    QTest::newRow("importStatements")
+            << "importStatements.qml"
+            << "importStatements.formatted.qml" << QStringList{} << RunOption::OnCopy;
+    QTest::newRow("arrayEndComma")
+            << "arrayEndComma.qml"
+            << "arrayEndComma.formatted.qml" << QStringList{} << RunOption::OnCopy;
+    QTest::newRow("escapeChars")
+            << "escapeChars.qml"
+            << "escapeChars.formatted.qml" << QStringList{} << RunOption::OnCopy;
+    QTest::newRow("javascriptBlock")
+            << "javascriptBlock.qml"
+            << "javascriptBlock.formatted.qml" << QStringList{} << RunOption::OnCopy;
+
+    //plainJS
+    QTest::newRow("nestedLambdaWithIfElse")
+            << "lambdaWithIfElseInsideLambda.js"
+            << "lambdaWithIfElseInsideLambda.formatted.js" << QStringList{} << RunOption::OnCopy;
+
+    QTest::newRow("indentEquals2")
+            << "threeFunctionsOneLine.js"
+            << "threeFunctions.formattedW2.js" << QStringList{"-w=2"} << RunOption::OnCopy;
+
+    QTest::newRow("tabIndents")
+            << "threeFunctionsOneLine.js"
+            << "threeFunctions.formattedTabs.js" << QStringList{"-t"} << RunOption::OnCopy;
+
+    QTest::newRow("normalizedFunctionSpacing")
+            << "threeFunctionsOneLine.js"
+            << "threeFunctions.formattedFuncSpacing.js"
+            << QStringList{ "-n", "--functions-spacing" } << RunOption::OnCopy;
+
+    QTest::newRow("esm_tabIndents")
+            << "mini_esm.mjs"
+            << "mini_esm.formattedTabs.mjs" << QStringList{ "-t" } << RunOption::OnCopy;
 }
 
 void TestQmlformat::testFormat()
@@ -350,7 +421,82 @@ void TestQmlformat::testFormat()
     QFETCH(QStringList, args);
     QFETCH(RunOption, runOption);
 
-    QCOMPARE(runQmlformat(testFile(file), args, true, runOption), readTestFile(fileFormatted));
+    auto formatted = runQmlformat(testFile(file), args, true, runOption, fileExt(file));
+    QEXPECT_FAIL("normalizedFunctionSpacing",
+                 "Normalize && function spacing are not yet supported for JS", Abort);
+    auto exp = readTestFile(fileFormatted);
+    QCOMPARE(formatted, exp);
+}
+
+void TestQmlformat::plainJS_data()
+{
+    QTest::addColumn<QString>("file");
+    QTest::addColumn<QString>("fileFormatted");
+
+    QTest::newRow("simpleStatement") << "simpleJSStatement.js"
+                                     << "simpleJSStatement.formatted.js";
+    QTest::newRow("simpleFunction") << "simpleOnelinerJSFunc.js"
+                                    << "simpleOnelinerJSFunc.formatted.js";
+    QTest::newRow("simpleLoop") << "simpleLoop.js"
+                                << "simpleLoop.formatted.js";
+    QTest::newRow("messyIfStatement") << "messyIfStatement.js"
+                                      << "messyIfStatement.formatted.js";
+    QTest::newRow("lambdaFunctionWithLoop") << "lambdaFunctionWithLoop.js"
+                                            << "lambdaFunctionWithLoop.formatted.js";
+    QTest::newRow("lambdaWithIfElse") << "lambdaWithIfElse.js"
+                                      << "lambdaWithIfElse.formatted.js";
+    QTest::newRow("nestedLambdaWithIfElse") << "lambdaWithIfElseInsideLambda.js"
+                                            << "lambdaWithIfElseInsideLambda.formatted.js";
+    QTest::newRow("twoFunctions") << "twoFunctions.js"
+                                  << "twoFunctions.formatted.js";
+    QTest::newRow("pragma") << "pragma.js"
+                            << "pragma.formatted.js";
+    QTest::newRow("classConstructor") << "class.js"
+                                      << "class.formatted.js";
+    QTest::newRow("legacyDirectives") << "directives.js"
+                                      << "directives.formatted.js";
+    QTest::newRow("legacyDirectivesWithComments") << "directivesWithComments.js"
+                                                  << "directivesWithComments.formatted.js";
+}
+
+void TestQmlformat::plainJS()
+{
+    QFETCH(QString, file);
+    QFETCH(QString, fileFormatted);
+
+    bool wasSuccessful;
+    LineWriterOptions opts;
+#ifdef Q_OS_WIN
+    opts.lineEndings = QQmlJS::Dom::LineWriterOptions::LineEndings::Windows;
+#endif
+    QString output = formatInMemory(testFile(file), &wasSuccessful, opts, WriteOutCheck::None);
+
+    QVERIFY(wasSuccessful && !output.isEmpty());
+
+    // TODO(QTBUG-119404)
+    QEXPECT_FAIL("classConstructor", "see QTBUG-119404", Abort);
+    // TODO(QTBUG-119770)
+    QEXPECT_FAIL("legacyDirectivesWithComments", "see QTBUG-119770", Abort);
+    auto exp = readTestFile(fileFormatted);
+    QCOMPARE(output, exp);
+}
+
+void TestQmlformat::ecmascriptModule()
+{
+    QString file("esm.mjs");
+    QString formattedFile("esm.formatted.mjs");
+
+    bool wasSuccessful;
+    LineWriterOptions opts;
+#ifdef Q_OS_WIN
+    opts.lineEndings = QQmlJS::Dom::LineWriterOptions::LineEndings::Windows;
+#endif
+    QString output = formatInMemory(testFile(file), &wasSuccessful, opts, WriteOutCheck::None);
+
+    QVERIFY(wasSuccessful && !output.isEmpty());
+
+    auto exp = readTestFile(formattedFile);
+    QCOMPARE(output, readTestFile(formattedFile));
 }
 
 #if !defined(QTEST_CROSS_COMPILED) // sources not available when cross compiled
@@ -454,12 +600,79 @@ void TestQmlformat::testBackupFileLimit()
     };
 }
 
+void TestQmlformat::testFilesOption_data()
+{
+    QTest::addColumn<QString>("containerFile");
+    QTest::addColumn<QStringList>("individualFiles");
+
+    QTest::newRow("initial") << "fileListToFormat"
+            << QStringList{"valid1.qml", "invalidEntry:cannot be parsed", "valid2.qml"};
+}
+
+void TestQmlformat::testFilesOption()
+{
+    QFETCH(QString, containerFile);
+    QFETCH(QStringList, individualFiles);
+
+    // Create a temporary directory
+    QTemporaryDir tempDir;
+    tempDir.setAutoRemove(false);
+    QStringList actualFormattedFilesPath;
+
+    // Iterate through files in the source directory and copy them to the temporary directory
+    const auto sourceDir = dataDirectory() + QDir::separator() + "filesOption";
+
+    // Create a file that contains the list of files to be formatted
+    const QString tempFilePath = tempDir.path() + QDir::separator() + containerFile;
+    QFile container(tempFilePath);
+    if (container.open(QIODevice::Text | QIODevice::WriteOnly)) {
+        QTextStream out(&container);
+
+        for (const auto &file : individualFiles) {
+            QString destinationFilePath = tempDir.path() + QDir::separator() + file;
+            if (QFile::copy(sourceDir + QDir::separator() + file, destinationFilePath))
+                actualFormattedFilesPath << destinationFilePath;
+            out << destinationFilePath << "\n";
+        }
+
+        container.close();
+    } else {
+        QFAIL("Cannot create temp test file\n");
+        return;
+    }
+
+    {
+        QProcess process;
+        process.start(m_qmlformatPath, QStringList{"-F", tempFilePath});
+        QVERIFY(process.waitForFinished());
+        QCOMPARE(process.exitStatus(), QProcess::NormalExit);
+    }
+
+    const auto readFile = [](const QString &filePath){
+        QFile file(filePath);
+        if (!file.open(QIODevice::ReadOnly)) {
+            qWarning() << "Error on opening the file " << filePath;
+            return QByteArray{};
+        }
+
+        return file.readAll();
+    };
+
+    for (const auto &filePath : actualFormattedFilesPath) {
+        auto expectedFormattedFile = QFileInfo(filePath).fileName();
+        const auto expectedFormattedFilePath = sourceDir + QDir::separator() +
+            expectedFormattedFile.replace(".qml", ".formatted.qml");
+
+        QCOMPARE(readFile(filePath), readFile(expectedFormattedFilePath));
+    }
+}
+
 QString TestQmlformat::runQmlformat(const QString &fileToFormat, QStringList args,
-                                    bool shouldSucceed, RunOption rOptions)
+                                    bool shouldSucceed, RunOption rOptions, QStringView ext)
 {
     // Copy test file to temporary location
     QTemporaryDir tempDir;
-    const QString tempFile = tempDir.path() + QDir::separator() + "to_format.qml";
+    const QString tempFile = (tempDir.path() + QDir::separator() + "to_format") % ext;
 
     if (rOptions == RunOption::OnCopy) {
         QFile::copy(fileToFormat, tempFile);
@@ -483,7 +696,8 @@ QString TestQmlformat::runQmlformat(const QString &fileToFormat, QStringList arg
 
     QFile temp(tempFile);
 
-    temp.open(QIODevice::ReadOnly);
+    if (!temp.open(QIODevice::ReadOnly))
+        qFatal("Could not open %s", qPrintable(tempFile));
     QString formatted = QString::fromUtf8(temp.readAll());
 
     return formatted;
@@ -493,19 +707,17 @@ QString TestQmlformat::formatInMemory(const QString &fileToFormat, bool *didSucc
                                       LineWriterOptions options, WriteOutChecks extraChecks,
                                       WriteOutChecks largeChecks)
 {
-    DomItem env = DomEnvironment::create(
+    auto env = DomEnvironment::create(
             QStringList(), // as we load no dependencies we do not need any paths
             QQmlJS::Dom::DomEnvironment::Option::SingleThreaded
                     | QQmlJS::Dom::DomEnvironment::Option::NoDependencies);
     DomItem tFile;
-    env.loadFile(
-            FileToLoad::fromFileSystem(env.ownerAs<DomEnvironment>(), fileToFormat),
-            [&tFile](Path, const DomItem &, const DomItem &newIt) { tFile = newIt; },
-            LoadOption::DefaultLoad);
-    env.loadPendingDependencies();
+    env->loadFile(FileToLoad::fromFileSystem(env, fileToFormat),
+                  [&tFile](Path, const DomItem &, const DomItem &newIt) { tFile = newIt; });
+    env->loadPendingDependencies();
     MutableDomItem myFile = tFile.field(Fields::currentItem);
 
-    DomItem writtenOut;
+    bool writtenOut;
     QString resultStr;
     if (myFile.field(Fields::isValid).value().toBool()) {
         WriteOutChecks checks = extraChecks;
@@ -523,7 +735,7 @@ QString TestQmlformat::formatInMemory(const QString &fileToFormat, bool *didSucc
         res.flush();
     }
     if (didSucceed)
-        *didSucceed = bool(writtenOut);
+        *didSucceed = writtenOut;
     return resultStr;
 }
 
