@@ -1,5 +1,5 @@
 // Copyright (C) 2018 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #ifndef TST_QMLLSMODULES_H
 #define TST_QMLLSMODULES_H
@@ -58,10 +58,15 @@ private slots:
     void linting();
     void rangeFormatting_data();
     void rangeFormatting();
-    void qmldirImportsFromBuild();
-    void qmldirImportsFromSource();
+    void qmldirImports_data();
+    void qmldirImports();
     void quickFixes_data();
     void quickFixes();
+    void automaticSemicolonInsertionForCompletions_data();
+    void automaticSemicolonInsertionForCompletions();
+    void hover_data();
+    void hover();
+    void checkQuickSnippets();
 
 private:
     QProcess m_server;
