@@ -2,6 +2,7 @@
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include <QtQmlToolingSettings/private/qqmltoolingsettings_p.h>
+#include <QtQmlToolingSettings/private/qqmltoolingutils_p.h>
 
 #include <QtQmlCompiler/private/qqmljsresourcefilemapper_p.h>
 #include <QtQmlCompiler/private/qqmljscompiler_p.h>
@@ -122,6 +123,11 @@ All warnings can be set to three levels:
     const QString qmlImportPathsSetting = QLatin1String("AdditionalQmlImportPaths");
     settings.addOption(qmlImportPathsSetting);
 
+    QCommandLineOption environmentOption(
+            QStringList() << "E",
+            QLatin1String("Use the QML_IMPORT_PATH environment variable to look for QML Modules"));
+    parser.addOption(environmentOption);
+
     QCommandLineOption qmlImportNoDefault(
                 QStringList() << "bare",
                 QLatin1String("Do not include default import directories or the current directory. "
@@ -296,7 +302,7 @@ All warnings can be set to three levels:
     QStringList defaultQmldirFiles;
     if (parser.isSet(qmldirFilesOption)) {
         defaultQmldirFiles = parser.values(qmldirFilesOption);
-    } else {
+    } else if (!parser.isSet(qmlImportNoDefault)){
         // If nothing given explicitly, use the qmldir file from the current directory.
         QFileInfo qmldirFile(QStringLiteral("qmldir"));
         if (qmldirFile.isFile()) {
@@ -360,64 +366,90 @@ All warnings can be set to three levels:
     QJsonArray jsonFiles;
 
     for (const QString &filename : positionalArguments) {
-        if (!parser.isSet(ignoreSettings)) {
+        if (!parser.isSet(ignoreSettings))
             settings.search(filename);
-            updateLogLevels();
+        updateLogLevels();
 
-            const QDir fileDir = QFileInfo(filename).absoluteDir();
-            auto addAbsolutePaths = [&](QStringList &list, const QStringList &entries) {
-                for (const QString &file : entries)
-                    list << (QFileInfo(file).isAbsolute() ? file : fileDir.filePath(file));
-            };
+        const QDir fileDir = QFileInfo(filename).absoluteDir();
+        auto addAbsolutePaths = [&](QStringList &list, const QStringList &entries) {
+            for (const QString &file : entries)
+                list << (QFileInfo(file).isAbsolute() ? file : fileDir.filePath(file));
+        };
 
-            resourceFiles = defaultResourceFiles;
+        resourceFiles = defaultResourceFiles;
 
-            addAbsolutePaths(resourceFiles, settings.value(resourceSetting).toStringList());
+        addAbsolutePaths(resourceFiles, settings.value(resourceSetting).toStringList());
 
-            qmldirFiles = defaultQmldirFiles;
-            if (settings.isSet(qmldirFilesSetting)
-                && !settings.value(qmldirFilesSetting).toStringList().isEmpty()) {
-                qmldirFiles = {};
-                addAbsolutePaths(qmldirFiles,
-                                 settings.value(qmldirFilesSetting).toStringList());
-            }
-
-            if (parser.isSet(qmlImportNoDefault)
-                || (settings.isSet(qmlImportNoDefaultSetting)
-                    && settings.value(qmlImportNoDefaultSetting).toBool())) {
-                qmlImportPaths = {};
-            } else {
-                qmlImportPaths = defaultImportPaths;
-            }
-
-            if (parser.isSet(qmlImportPathsOption))
-                qmlImportPaths << parser.values(qmlImportPathsOption);
-
-            addAbsolutePaths(qmlImportPaths, settings.value(qmlImportPathsSetting).toStringList());
-
-            QSet<QString> disabledPlugins;
-
-            if (parser.isSet(pluginsDisable)) {
-                for (const QString &plugin : parser.values(pluginsDisable))
-                    disabledPlugins << plugin.toLower();
-            }
-
-            if (settings.isSet(pluginsDisableSetting)) {
-                for (const QString &plugin : settings.value(pluginsDisableSetting).toStringList())
-                    disabledPlugins << plugin.toLower();
-            }
-
-            linter.setPluginsEnabled(!disabledPlugins.contains("all"));
-
-            if (!linter.pluginsEnabled())
-                continue;
-
-            auto &plugins = linter.plugins();
-
-            for (auto &plugin : plugins)
-                plugin.setEnabled(!disabledPlugins.contains(plugin.name().toLower()));
+        qmldirFiles = defaultQmldirFiles;
+        if (settings.isSet(qmldirFilesSetting)
+            && !settings.value(qmldirFilesSetting).toStringList().isEmpty()) {
+            qmldirFiles = {};
+            addAbsolutePaths(qmldirFiles, settings.value(qmldirFilesSetting).toStringList());
         }
 
+        if (parser.isSet(qmlImportNoDefault)
+            || (settings.isSet(qmlImportNoDefaultSetting)
+                && settings.value(qmlImportNoDefaultSetting).toBool())) {
+            qmlImportPaths = {};
+        } else {
+            qmlImportPaths = defaultImportPaths;
+        }
+
+        if (parser.isSet(qmlImportPathsOption))
+            qmlImportPaths << parser.values(qmlImportPathsOption);
+        if (parser.isSet(environmentOption)) {
+            if (silent) {
+                qmlImportPaths << qEnvironmentVariable("QML_IMPORT_PATH")
+                                          .split(QDir::separator(), Qt::SkipEmptyParts)
+                               << qEnvironmentVariable("QML2_IMPORT_PATH")
+                                          .split(QDir::separator(), Qt::SkipEmptyParts);
+            } else {
+                if (const QStringList dirsFromEnv =
+                            QQmlToolingUtils::getAndWarnForInvalidDirsFromEnv(u"QML_IMPORT_PATH"_s);
+                    !dirsFromEnv.isEmpty()) {
+                    qInfo().nospace().noquote()
+                            << "Using import directories passed from environment variable "
+                               "\"QML_IMPORT_PATH\": \""
+                            << dirsFromEnv.join(u"\", \""_s) << "\".";
+                    qmlImportPaths << dirsFromEnv;
+                }
+                if (const QStringList dirsFromEnv =
+                            QQmlToolingUtils::getAndWarnForInvalidDirsFromEnv(
+                                    u"QML2_IMPORT_PATH"_s);
+                    !dirsFromEnv.isEmpty()) {
+                    qInfo().nospace().noquote() << "Using import directories passed from the "
+                                                   "deprecated environment variable "
+                                                   "\"QML2_IMPORT_PATH\": \""
+                                                << dirsFromEnv.join(u"\", \""_s) << "\".";
+                    qmlImportPaths << dirsFromEnv;
+                }
+            }
+        }
+
+        addAbsolutePaths(qmlImportPaths, settings.value(qmlImportPathsSetting).toStringList());
+
+        QSet<QString> disabledPlugins;
+
+        if (parser.isSet(pluginsDisable)) {
+            for (const QString &plugin : parser.values(pluginsDisable))
+                disabledPlugins << plugin.toLower();
+        }
+
+        if (settings.isSet(pluginsDisableSetting)) {
+            for (const QString &plugin : settings.value(pluginsDisableSetting).toStringList())
+                disabledPlugins << plugin.toLower();
+        }
+
+        linter.setPluginsEnabled(!disabledPlugins.contains("all"));
+
+        if (!linter.pluginsEnabled())
+            continue;
+
+        auto &plugins = linter.plugins();
+
+        for (auto &plugin : plugins)
+            plugin.setEnabled(!disabledPlugins.contains(plugin.name().toLower()));
+
         const bool isFixing = parser.isSet(fixFile);
 
         QQmlJSLinter::LintResult lintResult;
@@ -512,8 +544,10 @@ All warnings can be set to three levels:
             QTextStream(stdout) << QString::fromUtf8(json);
         } else {
             QFile file(fileName);
-            file.open(QFile::WriteOnly);
-            file.write(json);
+            if (file.open(QFile::WriteOnly))
+                file.write(json);
+            else
+                success = false;
         }
     }
 
